/*
 * Licensed to STRATIO (C) under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership.  The STRATIO (C) licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package com.stratio.meta.core.structures;


public class InnerJoin {

  private String keyspace = null;

  private boolean keyspaceInc = false;

  private String tableName;

  private SelectorIdentifier leftField;

  private SelectorIdentifier rightField;

  private InnerJoin(String tableName) {

    this.tableName = tableName;

    if (this.tableName.contains(".")) {
      String[] ksAndTablename = this.tableName.split("\\.");
      keyspace = ksAndTablename[0];
      this.tableName = ksAndTablename[1];
      keyspaceInc = true;
    }
  }

  public InnerJoin(String tableName, String leftField, String rightField) {

    this(tableName);

    this.leftField = new SelectorIdentifier(leftField);
    this.rightField = new SelectorIdentifier(rightField);
  }

  public InnerJoin(String tableName, SelectorIdentifier leftField, SelectorIdentifier rightField) {

    this(tableName);

    this.leftField = leftField;
    this.rightField = rightField;
  }

  public String getTablename() {
    return tableName;
  }

  public SelectorIdentifier getLeftField() {
    return leftField;
  }

  public SelectorIdentifier getRightField() {
    return rightField;
  }

  public String getKeyspace() {

    return keyspace;
  }

  /**
   * Returns the statement effective keyspace: if the join has a specific keyspace, that's the
   * effective keyspace; otherwise, the given parent statement keyspace.
   * 
   * @param sessionKeyspace
   * @return
   */
  public String findEffectiveKeyspace(String parentStatementKeyspace) {

    return keyspaceInc ? keyspace : parentStatementKeyspace;
  }

  @Override
  public String toString() {
    StringBuilder sb = new StringBuilder();
    if (keyspaceInc) {
      sb.append(keyspace);
      sb.append(".");
    }
    sb.append(tableName);
    sb.append(" ON ").append(leftField).append("=").append(rightField);
    return sb.toString();
  }

  public boolean isKeyspaceInc() {
    return keyspaceInc;
  }
}
