/*
 * Licensed to STRATIO (C) under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership.  The STRATIO (C) licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package com.stratio.meta.core.executor.command;

import com.stratio.meta.common.result.CommandResult;
import com.stratio.meta.common.result.QueryStatus;
import com.stratio.meta.common.result.Result;
import com.stratio.meta.core.executor.BasicExecutorTest;
import com.stratio.meta.core.statements.DropKeyspaceStatement;
import com.stratio.meta.core.statements.ExplainPlanStatement;
import com.stratio.meta.core.utils.*;
import org.testng.annotations.Test;

import static org.testng.Assert.assertEquals;

public class ExplainPlanStatementTest extends BasicExecutorTest {

    @Test
    public void executionForExplainPlanForDropKeyspace(){

        MetaQuery metaQuery = new MetaQuery("EXPLAIN PLAN FOR DROP KEYSPACE demo;");

        DropKeyspaceStatement dropStmt = new DropKeyspaceStatement("demo", false);

        ExplainPlanStatement stmt = new ExplainPlanStatement(dropStmt);

        stmt.validate(metadataManager, null);

        Tree tree = new Tree();
        tree.setNode(new MetaStep(MetaPath.COMMAND, stmt));
        metaQuery.setPlan(tree);
        metaQuery.setStatus(QueryStatus.PLANNED);

        Result result = validateOk(metaQuery, "executionForExplainPlanForDropKeyspace");

        String expectedPlan = "(CASSANDRA) Drop keyspace demo";

        String resultPlan = String.class.cast(((CommandResult) result).getResult());

        assertEquals(expectedPlan, resultPlan, "  RESULT PLAN: "+resultPlan+System.lineSeparator()+"EXPECTED PLAN: "+expectedPlan);
    }

}
